#include "AMDiS.h"

using namespace AMDiS;
using namespace std;

// ===========================================================================
// ===== function definitions ================================================
// ===========================================================================

/// Dirichlet boundary function
class G : public AbstractFunction<double, WorldVector<double> >
{
public:

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    return exp(-10.0 * (x * x));
  }
};

/// RHS function
class F : public AbstractFunction<double, WorldVector<double> >
{
public:

  /// Implementation of AbstractFunction::operator().
  double operator()(const WorldVector<double>& x) const 
  {
    int dow = x.getSize();
    double r2 = (x * x);
    double ux = exp(-10.0 * r2);
    return -(400.0 * r2 - 20.0 * dow) * ux;
  }
};

// ===========================================================================
// ===== main program ========================================================
// ===========================================================================

int main(int argc, char* argv[])
{
  FUNCNAME("main");

  // ===== check for init file =====
  TEST_EXIT(argc == 2)("usage: ellipt initfile\n");

  // ===== init parameters =====
  Parameters::init(true, argv[1]);

  // ===== create and init the scalar problem ===== 
  ProblemScal ellipt("ellipt");
  ellipt.initialize(INIT_ALL);

  // === create adapt info ===
  AdaptInfo *adaptInfo = new AdaptInfo("ellipt->adapt");

  // === create adapt ===
  AdaptStationary *adapt = new AdaptStationary("ellipt->adapt",
					       &ellipt,
					       adaptInfo);
  
  // ===== add boundary conditions =====
  ellipt.addDirichletBC(1, new G);

  // ===== create matrix operator =====
  Operator matrixOperator(Operator::MATRIX_OPERATOR, ellipt.getFESpace());
  matrixOperator.addSecondOrderTerm(new Laplace_SOT);
  ellipt.addMatrixOperator(&matrixOperator);

  // ===== create rhs operator =====
  Operator rhsOperator(Operator::VECTOR_OPERATOR, ellipt.getFESpace());
  rhsOperator.addZeroOrderTerm(new CoordsAtQP_ZOT(new F()));
  ellipt.addVectorOperator(&rhsOperator);

  // ===== start adaption loop =====
  adapt->adapt();

  ellipt.writeFiles(adaptInfo, true);
}


